import logging
from celery import Task
from celery import shared_task
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.http import HttpResponse
from django.template import Context
from django.template import TemplateDoesNotExist
from django.template.loader import get_template
from django.utils.decorators import method_decorator
from django.views.generic import TemplateView
from ucamlookup import createConnection, PersonMethods


LOGGER = logging.getLogger('common')


def get_users_email_address_from_lookup(user, email_only=False):
    """
    This function look's up an email address for a user. If one cannot be found it returns a default
    of crsid@cam.ac.uk.

    :param user: django user
    :param email_only: True = '{email}' False = '{name} <{email}>'
    :return: looked up user email of default
    """
    conn = createConnection()
    results = PersonMethods(conn).search(query=user.username, fetch="email")
    if user.get_full_name() and not email_only:
        default = "%s <%s@cam.ac.uk>" % (user.get_full_name(), user.username)
    else:
        default = "%s@cam.ac.uk" % user.username
    if results is None or len(results) == 0:
        LOGGER.info("no results returned from email lookup - defaulting to '%s'" % default)
        return default
    if results[0].attributes is None or len(results[0].attributes) == 0:
        LOGGER.warning("no attributes returned from email lookup - defaulting to '%s'" % default)
        return default
    if '@' not in results[0].attributes[0].value:
        LOGGER.warning("'%s' is not an email address - defaulting to '%s'" % (results[0].attributes[0].value, default))
        return default
    if user.get_full_name() and not email_only:
        return "%s <%s>" % (user.get_full_name(), results[0].attributes[0].value)
    return results[0].attributes[0].value


class TaskWithFailure(Task):
    """
    Abstract celery Task that logs failure.
    """
    abstract = True

    def on_failure(self, exc, task_id, args, kwargs, einfo):
        LOGGER.error("An error happened (%s) when trying to execute celery task with id %s and arguments %s and %s\n\n"
                     "The traceback is:\n%s\n", exc, task_id, args, kwargs, einfo)


class ProtectedView(TemplateView):
    """
    A abstract TemplateView to extend from that requires login.
    """
    abstract = True

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(ProtectedView, self).dispatch(*args, **kwargs)


@shared_task(base=TaskWithFailure)
def test_celery_email():
    """
    This function is used to test the logging and exception handling functionality of celery
    """
    LOGGER.info("This is a test Celery info log")
    LOGGER.warning("This is a test Celery warning log")
    LOGGER.error("This is a test Celery error log")
    raise Exception("This is test for a Celery Exception")


def simple_authorization(func):
    """
    Decorator to test an HTTP request for an authorization header with a matching .
    """
    def func_wrapper(*argv, **kwargs):

        request = argv[0]

        if 'HTTP_AUTHORIZATION' not in request.META:
            return HttpResponse('Unauthorized', status=401)

        parts = request.META['HTTP_AUTHORIZATION'].split(" ")
        if len(parts) != 2 or parts[0].lower() != 'bearer' or parts[1] != settings.SSGW_API_TOKEN:
            return HttpResponse('Unauthorized', status=401)

        return func(*argv, **kwargs)

    return func_wrapper


def paginate(request, object_list, per_page=25):
    """
    Helper method for django Paginator - assumes a request parameter of "page".

    :param request: http request
    :param object_list: sourse to select page from
    :param per_page: the number of objects per page
    :return: Paginator page
    """
    paginator = Paginator(object_list, per_page)
    page = request.GET.get('page')
    try:
        return paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        return paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        return paginator.page(paginator.num_pages)


def send(recipients, email_template, context, attachments=None, reply_to=None, bcc=False, **kwargs):
    """
    Sends an email. By convention the first line of the template is assumed to be the subject.

    :param recipients: either list of recipients or single recipient -
                       each receipient is either an email string or a User
    :param email_template: the email template
    :param context: context for the email template
    :param attachments: optional attachments (assumes either one of the other)
    :param reply_to: optional "reply to" address
    :param bcc: if True then bcc to SERVER_EMAIL

    :return: the sent EmailMessage
    """
    template = get_template('email/' + email_template + '.txt')
    subject_and_body = template.render(Context(context)).split('\n', 1)
    if isinstance(recipients, str) or isinstance(recipients, User):
        recipients = [recipients]

    to = [recipient if isinstance(recipient, str) else get_users_email_address_from_lookup(recipient)
          for recipient in recipients]

    if reply_to:
        if reply_to.__class__ == list:
            kwargs['reply_to'] = reply_to
        else:
            kwargs['reply_to'] = [reply_to]

    if bcc:
        kwargs['bcc'] = [settings.SERVER_EMAIL]

    final_kwargs = {**{
        'subject': subject_and_body[0],
        'body': subject_and_body[1],
        'from_email': settings.SERVER_EMAIL_FULL,
        'to': to,
        'headers': {'Return-Path': settings.SERVER_EMAIL}
    }, **kwargs}

    message = EmailMultiAlternatives(**final_kwargs)

    try:
        template = get_template('email/' + email_template + '.html')
        message.attach_alternative(template.render(Context(context)), "text/html")
    except TemplateDoesNotExist:
        pass

    if attachments:
        if attachments.__class__ != list:
            attachments = [attachments]
        for attachment in attachments:
            if attachment.__class__ == tuple:
                message.attachments.append(attachment)
            else:
                message.attach(attachment.name, attachment.read())

    message.send()
    LOGGER.info("email sent: to='%s' template='%s' context=%s" % (to, email_template, context))

    return message


def merge_dicts(dict1, dict2):
    """
    Merge 2 dictionaries.
    """
    merged = dict1.copy()
    merged.update(dict2)
    return merged



from django import template

register = template.Library()


@register.filter
def username_list(users):
    """
    :param users: list of django User entities
    :return: a comma seperated list of crsid's
    """
    return ",".join([user.username for user in users])


@register.filter
def display_user(user):
    """
    :param user: django User
    :return: a rendering of the user's name sand crsid
    """
    return "%s (%s)" % (user.get_full_name(), user.username)


@register.filter
def unique_entity_id(entity):
    """
    :param entity: django model
    :return: unique token combining the model type and id for use in HTML
    """
    return "%s-%s" % (type(entity).__name__, entity.id)



from collections import namedtuple

from django.contrib.auth.models import User
from django.test import TestCase
from mock import Mock, MagicMock

import automationcommon.utils as utils


class UtilsTests(TestCase):

    def setUp(self):

        def create_connection_fake():
            return None

        self.createConnection = utils.createConnection
        utils.createConnection = Mock(side_effect=create_connection_fake)

        self.PersonMethods_search = utils.PersonMethods.search

    def test_get_users_email_address_from_lookup(self):

        # named tuples to help with results
        Attributes = namedtuple('Attributes', 'attributes')
        Value = namedtuple('Value', 'value')

        # setup user where lookup fails
        user_none = User(username='jfk1000', last_name="John F Kennedy")
        utils.PersonMethods.search = MagicMock(return_value=None)

        # test for full address
        result = utils.get_users_email_address_from_lookup(user_none)
        self.assertEqual(result, "John F Kennedy <jfk1000@cam.ac.uk>")

        # test for email only
        result = utils.get_users_email_address_from_lookup(user_none, True)
        self.assertEqual(result, "jfk1000@cam.ac.uk")

        # setup user where lookup success
        user_found = User(username='fsf1000', last_name="F Scott Fitzgerald")
        utils.PersonMethods.search = MagicMock(return_value=[Attributes(attributes=[Value(value="fsf1001@cam.ac.uk")])])

        # test for full address
        result = utils.get_users_email_address_from_lookup(user_found)
        self.assertEqual(result, "F Scott Fitzgerald <fsf1001@cam.ac.uk>")

        # test for email only
        result = utils.get_users_email_address_from_lookup(user_found, True)
        self.assertEqual(result, "fsf1001@cam.ac.uk")

    def test_paginate(self):
        Request = namedtuple('Request', 'GET')
        object_list = [1, 2, 3, 4, 5, 6, 7]
        # 1st page
        paginator = utils.paginate(Request(GET={}), object_list, 3)
        self.assertEqual(paginator[0], 1)
        self.assertEqual(paginator[2], 3)
        # last page
        paginator = utils.paginate(Request(GET={"page": 3}), object_list, 3)
        self.assertEqual(paginator[0], 7)
        # out of range (last page)
        paginator = utils.paginate(Request(GET={"page": 4}), object_list, 3)
        self.assertEqual(paginator[0], 7)

    def tearDown(self):
        utils.createConnection = self.createConnection
        utils.PersonMethods.search = self.PersonMethods_search



from datetime import datetime
from bs4 import BeautifulSoup
from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test import override_settings
from mock import patch
from ucamwebauth.tests import create_wls_response


def assert_contains_in_order(actual, expected):
    """
    assert that all the strings in *expected* occur in the *actual* in the order in which they are listed

    :param actual: target string to search
    :param expected: collection of strings to search for
    """
    for query in expected:
        query = str(query)
        index = actual.find(query)
        if index == -1:
            raise AssertionError("'%s' not found in target" % query)
        else:
            actual = actual[index + len(query):]


user_dict = {
    'test0001': 'Testy Mike',
    'it123': 'Ivanna Tinkle',
    'ac123': 'Al Coholic',
    'hr123': 'Harry Rump',
    'bl123': 'Bill Loney',
    'jk123': 'Joe King'
}


PASSWORD = 'notsecret'


@override_settings(CELERY_EAGER_PROPAGATES_EXCEPTIONS=True, CELERY_ALWAYS_EAGER=True, BROKER_BACKEND='memory',
                   DEBUG=True)
class UnitTestCase(TestCase):

    def setUp(self):

        def return_visible_name_by_crsid_side_effect(*args):
            return user_dict[args[0]]

        return_visible_name_by_crsid = self.patch("ucamlookup.signals.return_visibleName_by_crsid")
        return_visible_name_by_crsid.side_effect = return_visible_name_by_crsid_side_effect

    def do_test_login(self, username, superuser=False):
        """
        Do a fake login.

        :param username: username
        :param superuser: whether or not a super user
        :return: logged in user
        """
        if username in user_dict:
            user = User.objects.create_superuser(username, "%s@cam.ac.uk" % username, PASSWORD) \
                if superuser else User.objects.create_user(username, password=PASSWORD)
            self.client.login(username=username, password=PASSWORD)
            return user

    def do_admin_login(self, username):
        """
        Do a fake adminlogin.

        :param username: username
        :return: logged in admin user
        """
        user = self.do_test_login(username)
        user.is_superuser = True
        user.save()
        return user

    def get_html_test_target(self, response):
        """
        Returns the test_target div for a valid html response encapsulated with BeautifulSoup.

        :param response: http response
        :return: a BeautifulSoup instance

        """
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response['Content-Type'], "text/html; charset=utf-8")
        return BeautifulSoup(response.content, 'html.parser').find("div", {"class": "test_target"})

    def assert_element_with_text(self, target, element, text):
        """
        assert that an HTML element with text can be found in target

        :param target: bs4 html tree
        :param element: element to find
        :param text: text in element to match
        """
        for el in target.find_all(element):
            if el.get_text().strip() == text:
                return
        self.fail("Couldn't find element '%s' with text '%s'" % (element, text))

    def patch(self, target_name):
        patcher = patch(target_name)
        target = patcher.start()
        self.addCleanup(patcher.stop)
        return target


def do_test_login(self, user="user1"):
    with self.settings(UCAMWEBAUTH_CERTS={901: """-----BEGIN CERTIFICATE-----
MIIDzTCCAzagAwIBAgIBADANBgkqhkiG9w0BAQQFADCBpjELMAkGA1UEBhMCR0Ix
EDAOBgNVBAgTB0VuZ2xhbmQxEjAQBgNVBAcTCUNhbWJyaWRnZTEgMB4GA1UEChMX
VW5pdmVyc2l0eSBvZiBDYW1icmlkZ2UxLTArBgNVBAsTJENvbXB1dGluZyBTZXJ2
aWNlIERFTU8gUmF2ZW4gU2VydmljZTEgMB4GA1UEAxMXUmF2ZW4gREVNTyBwdWJs
aWMga2V5IDEwHhcNMDUwNzI2MTMyMTIwWhcNMDUwODI1MTMyMTIwWjCBpjELMAkG
A1UEBhMCR0IxEDAOBgNVBAgTB0VuZ2xhbmQxEjAQBgNVBAcTCUNhbWJyaWRnZTEg
MB4GA1UEChMXVW5pdmVyc2l0eSBvZiBDYW1icmlkZ2UxLTArBgNVBAsTJENvbXB1
dGluZyBTZXJ2aWNlIERFTU8gUmF2ZW4gU2VydmljZTEgMB4GA1UEAxMXUmF2ZW4g
REVNTyBwdWJsaWMga2V5IDEwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBALhF
i9tIZvjYQQRfOzP3cy5ujR91ZntQnQehldByHlchHRmXwA1ot/e1WlHPgIjYkFRW
lSNcSDM5r7BkFu69zM66IHcF80NIopBp+3FYqi5uglEDlpzFrd+vYllzw7lBzUnp
CrwTxyO5JBaWnFMZrQkSdspXv89VQUO4V4QjXV7/AgMBAAGjggEHMIIBAzAdBgNV
HQ4EFgQUgjC6WtA4jFf54kxlidhFi8w+0HkwgdMGA1UdIwSByzCByIAUgjC6WtA4
jFf54kxlidhFi8w+0HmhgaykgakwgaYxCzAJBgNVBAYTAkdCMRAwDgYDVQQIEwdF
bmdsYW5kMRIwEAYDVQQHEwlDYW1icmlkZ2UxIDAeBgNVBAoTF1VuaXZlcnNpdHkg
b2YgQ2FtYnJpZGdlMS0wKwYDVQQLEyRDb21wdXRpbmcgU2VydmljZSBERU1PIFJh
dmVuIFNlcnZpY2UxIDAeBgNVBAMTF1JhdmVuIERFTU8gcHVibGljIGtleSAxggEA
MAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEEBQADgYEAsdyB+9szctHHIHE+S2Kg
LSxbGuFG9yfPFIqaSntlYMxKKB5ba/tIAMzyAOHxdEM5hi1DXRsOok3ElWjOw9oN
6Psvk/hLUN+YfC1saaUs3oh+OTfD7I4gRTbXPgsd6JgJQ0TQtuGygJdaht9cRBHW
wOq24EIbX5LquL9w+uvnfXw=
-----END CERTIFICATE-----"""}):
        self.client.get(reverse('raven_return'),
                        {'WLS-Response': create_wls_response(raven_issue=datetime.utcnow().strftime('%Y%m%dT%H%M%SZ'),
                                                             raven_url=settings.UCAMWEBAUTH_RETURN_URL,
                                                             raven_principal=user)})
        self.assertIn('_auth_user_id', self.client.session)



from django.contrib.auth.models import User

import automationcommon.templatetags.custom_filters as custom_filters
from automationcommon.tests.utils import UnitTestCase


class CustomFilterTests(UnitTestCase):

    def test_username_list(self):
        self.assertEqual(custom_filters.username_list(User.objects.all()), "")
        User.objects.create(username="bl123")
        self.assertEqual(custom_filters.username_list(User.objects.all()), "bl123")
        User.objects.create(username="it123")
        self.assertEqual(custom_filters.username_list(User.objects.all()), "bl123,it123")

    def test_display_user(self):
        user = User.objects.create(username="bl123")
        self.assertEqual(custom_filters.display_user(user), "Bill Loney (bl123)")

    def test_unique_entity_id(self):
        user = User.objects.create(username="bl123")
        self.assertEqual(custom_filters.unique_entity_id(user), "User-%s" % user.id)



from django import template
from django.conf import settings

register = template.Library()


@register.simple_tag(takes_context=True)
def hijack_notification_proxy(context):
    """
    This tag only renders hijack_notification if the library is present
    """
    if hasattr(settings, 'IMPERSONATION_APPS'):
        from hijack.templatetags.hijack_tags import hijack_notification
        return hijack_notification(context)

    return ''



from django.contrib.auth.models import User
from django.db import models


class Creatable(models.Model):
    """
    An abstract Model encapsulating an entity that can be created
    """

    # creator of the entity
    creator = models.ForeignKey(User, related_name="+")
    # when the entity was created
    creation_date = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True



from django.conf import settings
from django.conf.urls import url, include
from automationcommon import views as common


urlpatterns = [
    # service status page
    url(r'^status/20d47308-dd08-4aa6-991c-c46a6e7fced7/$', common.status, name='status-page'),
]


if hasattr(settings, 'IMPERSONATION_APPS'):
    urlpatterns += [
        url(r'^hijack/', include('hijack.urls')),
        url(r'^superadmin/impersonate', common.impersonate),
    ]



import logging
import requests
from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.contrib.auth.models import User
from django.shortcuts import render


LOGGER = logging.getLogger('common')


@user_passes_test(lambda user: user.is_superuser)
def impersonate(request):
    """
    Renders the page allow an admin to impersonate any user.
    """
    return render(request, 'impersonate.html')


def status(request):
    """
    Checks that all the services (external or internal) used by the Self-Service Gateway are working,
    returning a HTTP 500 if any of them do not work, or a 200 otherwise. It contains individual services checklist
    in the view.
    """

    status_results = {}
    try:
        User.objects.first()
        status_results['Database'] = True
    except:
        status_results['Database'] = False

    overall_result = status_results['Database']

    if hasattr(settings, 'SERVICE_CHECKS'):
        for name, url in settings.SERVICE_CHECKS.items():
            try:
                response = requests.get(url)
                status_results[name] = response.status_code == 200
            except Exception as e:
                status_results[name] = False
                overall_result = overall_result and status_results[name]

    if not overall_result:
        LOGGER.error("Status Webpage returning 500: %s" % status_results)

    return render(request, 'status.html', {'status_results': status_results}, status=200 if overall_result else 500)



#!/usr/bin/env python

from distutils.core import setup

from pip.req import parse_requirements
from setuptools import find_packages

setup(
    name='django-automationcommon',
    # When changing this version number, remember to update CHANGELOG.
    version='1.0',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description='Common functionality across different Django projects of the UofC UIS Automation team',
    long_description=open('README.rst').read(),
    url='https://git.csx.cam.ac.uk/i/ucs/automation/django-automationcommon',
    author='Automation team, University Information Services, University of Cambridge',
    author_email='amc203@cam.ac.uk',
    install_requires=[
        'django>=1.8,<1.12',
        'django-ucamlookup>!=1.9.2',
        'requests',
        'celery<4'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)
